import 'package:flutter/material.dart';

// ignore_for_file: must_be_immutable, annotate_overrides, overridden_fields
class AdeptIconButton extends StatefulWidget {
  final Key? key;
  ValueNotifier<bool>? controller;
  final double iconSize;
  final EdgeInsetsGeometry padding;
  final AlignmentGeometry alignment;
  final Color? focusColor;
  final Color? hoverColor;
  final Color? color;
  final Color? splashColor;
  final Color? highlightColor;
  final Color? disabledColor;
  final VoidCallback onPressed;
  final FocusNode? focusNode;
  final String? tooltip;
  Widget icon;
  Widget? iconOn, iconOff;

  AdeptIconButton({
    this.key,
    this.controller,
    this.iconSize = 24.0,
    this.padding = const EdgeInsets.all(8.0),
    this.alignment = Alignment.center,
    required this.icon,
    this.iconOn,
    this.iconOff,
    this.color,
    this.focusColor,
    this.hoverColor,
    this.highlightColor,
    this.splashColor,
    this.disabledColor,
    required this.onPressed,
    this.focusNode,
    this.tooltip,
  }) {
    controller ??= ValueNotifier(false);
    iconOff ??= icon;
  }

  @override
  AdeptIconButtonState createState() => AdeptIconButtonState();
}

class AdeptIconButtonState extends State<AdeptIconButton> {
  @override
  void initState() {
    super.initState();
    widget.controller?.addListener(onCheckedChanged);
  }

  void onCheckedChanged() {
    setState(() {});
  }

  Icon? getIcon() {
    if (widget.iconOn != null && widget.iconOff != null) {
      return (widget.controller?.value ?? false)
          ? widget.iconOn as Icon?
          : widget.iconOff as Icon?;
    }
    return widget.icon as Icon?;
  }

  void onPressed() {
    widget.controller!.value = !widget.controller!.value;
    widget.onPressed();
  }

  @override
  Widget build(BuildContext context) {
    return IconButton(
      icon: getIcon()!,
      onPressed: onPressed,
      key: widget.key,
      padding: widget.padding,
      color: widget.color,
      alignment: widget.alignment,
      tooltip: widget.tooltip,
      disabledColor: widget.disabledColor,
      focusColor: widget.focusColor,
      focusNode: widget.focusNode,
      highlightColor: widget.highlightColor,
      hoverColor: widget.hoverColor,
      iconSize: widget.iconSize,
      splashColor: widget.splashColor,
    );
  }

  @override
  void dispose() {
    widget.controller?.removeListener(onCheckedChanged);
    super.dispose();
  }
}
