import 'package:flutter/material.dart';
import 'package:shimmer/shimmer.dart';

class AdeptShimmer<T> extends StatelessWidget {
  final Key? _key;
  final Widget? child;
  final Widget? shimmerChild;
  final Duration period;
  final ShimmerDirection direction;
  final Color? baseColor;
  final Color? highlightColor;
  final Gradient gradient;
  final int loop;
  final bool enabled;

  AdeptShimmer({
    Key? key,
    this.child,
    this.shimmerChild,
    this.baseColor,
    this.highlightColor,
    Gradient? gradient,
    this.period = const Duration(milliseconds: 1500),
    this.direction = ShimmerDirection.ltr,
    this.loop = 0,
    this.enabled = true,
  })  : _key = key,
        gradient = gradient ??
            LinearGradient(
                begin: Alignment.topLeft,
                end: Alignment.centerRight,
                colors: <Color>[
                  baseColor ?? Colors.grey.shade300,
                  baseColor ?? Colors.grey.shade300,
                  highlightColor ?? Colors.grey.shade100,
                  baseColor ?? Colors.grey.shade300,
                  baseColor ?? Colors.grey.shade300,
                ],
                stops: const <double>[
                  0.0,
                  0.35,
                  0.5,
                  0.65,
                  1.0
                ]);

  @override
  Widget build(BuildContext context) {
    return AnimatedSwitcher(
      duration: const Duration(milliseconds: 300),
      child: enabled
          ? Shimmer(
              key: _key,
              gradient: gradient,
              direction: direction,
              period: period,
              loop: loop,
              enabled: enabled,
              child: shimmerChild ?? child ?? Container(),
            )
          : child ?? Container(),
    );
  }

  static Container roundCornerShimmerChild({
    double? width,
    double? height,
    double? radius,
    EdgeInsetsGeometry? padding,
    EdgeInsetsGeometry? margin,
  }) =>
      Container(
        padding: padding,
        margin: margin,
        width: width,
        height: height,
        decoration: BoxDecoration(
          borderRadius: BorderRadius.all(Radius.circular(radius ?? 16)),
          color: Colors.white,
        ),
      );
}
