import 'package:flutter/material.dart';
import 'package:adeptutils/adeptutils.dart';

// ignore_for_file: must_be_immutable, annotate_overrides, overridden_fields
class AdeptDropDownButton<T> extends StatefulWidget {
  AdeptValueController<T>? controller;
  String? label;
  bool useTextUnspecified;
  String? textUnspecified;
  List<String?>? entries;
  List<T?>? entryValues;

  Key? key;
  T? value;
  Widget? hint;
  Widget? disabledHint;
  ValueChanged<T>? onChanged;
  int elevation;
  TextStyle? style;
  Widget? underline;
  Widget? icon;
  Color? iconDisabledColor;
  Color? iconEnabledColor;
  double iconSize;
  bool isDense;
  bool isExpanded;

  AdeptDropDownButton({
    this.controller,
    this.label,
    this.useTextUnspecified = true,
    this.textUnspecified,
    this.entries,
    this.entryValues,
    this.key,
    this.value,
    this.hint,
    this.disabledHint,
    this.onChanged,
    this.elevation = 8,
    this.style,
    this.underline,
    this.icon,
    this.iconDisabledColor,
    this.iconEnabledColor,
    this.iconSize = 24.0,
    this.isDense = false,
    this.isExpanded = false,
  }) : super() {
    assert(entries != null);
    assert(entryValues != null);
    assert(entries!.length == entryValues!.length);
    textUnspecified ??= "Unspecified";
    label ??= "";
    if (useTextUnspecified) {
      entries!.insert(0, textUnspecified);
      entryValues!.insert(0, null);
    }
    controller ??= AdeptValueController(value: entryValues![0]);
  }

  @override
  State createState() => _AdeptDropDownButtonState<T>();
}

class _AdeptDropDownButtonState<T> extends State<AdeptDropDownButton> {
  Color? colorPrimary, colorTextTertiary;
  List<DropdownMenuItem<T>>? items;

  @override
  void initState() {
    super.initState();
    init();
    listeners();
  }

  void init() {
    generateFilterItems();
  }

  void generateFilterItems() {
    items = [];
    for (int i = 0; i < widget.entries!.length; ++i) {
      items!.add(DropdownMenuItem<T>(
        value: widget.entryValues![i],
        child: Text(widget.entries![i]!),
      ));
    }
  }

  void listeners() {
    widget.controller!.addListener(onValueChanged);
  }

  void onValueChanged() {
    if (widget.onChanged != null) widget.onChanged!(widget.controller!.value);
  }

  void checkColors(context) {
    colorPrimary ??= Theme.of(context).primaryColor;
    colorTextTertiary ??= Colors.grey.shade600;
  }

  @override
  Widget build(BuildContext context) {
    checkColors(context);
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Text(
          widget.label!,
          style: TextStyle(
              color: widget.controller!.value != null
                  ? colorPrimary
                  : colorTextTertiary,
              fontSize: 12),
        ),
        Theme(
          data: ThemeData(brightness: Theme.of(context).brightness),
          child: DropdownButton<T>(
            key: widget.key,
            isExpanded: widget.isExpanded,
            icon: widget.icon,
            style: widget.style,
            iconSize: widget.iconSize,
            elevation: widget.elevation,
            hint: widget.hint,
            isDense: widget.isDense,
            disabledHint: widget.disabledHint,
            iconDisabledColor: widget.iconDisabledColor,
            iconEnabledColor: widget.iconEnabledColor,
            underline: widget.underline,
            items: items,
            value: widget.controller!.value,
            onChanged: (value) {
              widget.controller!.value = value;
              setState(() {});
            },
          ),
        ),
        !widget.controller!.hasError
            ? Container()
            : Text(
                widget.controller!.error!,
                style: TextStyle(
                    color: Theme.of(context).colorScheme.error, fontSize: 10),
              ),
      ],
    );
  }

  @override
  void dispose() {
    widget.controller!.removeListener(onValueChanged);
    super.dispose();
  }
}
