import 'package:flutter/material.dart';

class SwipeView extends StatelessWidget {
  final Widget child;
  final VoidCallback? onSwipeLeft;
  final VoidCallback? onSwipeRight;
  final VoidCallback? onSwipeTop;
  final VoidCallback? onSwipeBottom;
  final GestureTapCallback? onTap;
  final HitTestBehavior behavior;

  const SwipeView({
    required this.child,
    this.onSwipeLeft,
    this.onSwipeRight,
    this.onSwipeTop,
    this.onSwipeBottom,
    this.onTap,
    this.behavior = HitTestBehavior.opaque,
  });

  bool get listenForHorizontalGestures =>
      onSwipeLeft != null || onSwipeRight != null;

  bool get listenForVerticalGestures =>
      onSwipeTop != null || onSwipeBottom != null;

  @override
  Widget build(BuildContext context) {
    double gestureStartX = 0, gestureEndX = 0;
    double gestureStartY = 0, gestureEndY = 0;
    double gestureDeltaHorizontal = 0, gestureDeltaVertical = 0;
    final deltaThreshold = MediaQuery.of(context).size.shortestSide * 30 / 100;
    const velocityThreshold = 200; //pixels per second
    return GestureDetector(
      onHorizontalDragStart: !listenForHorizontalGestures
          ? null
          : (details) {
              gestureStartX = details.globalPosition.dx;
            },
      onHorizontalDragUpdate: !listenForHorizontalGestures
          ? null
          : (details) {
              gestureEndX = details.globalPosition.dx;
              gestureDeltaHorizontal = gestureEndX - gestureStartX;
            },
      onHorizontalDragEnd: !listenForHorizontalGestures
          ? null
          : (details) {
              final velocity = details.primaryVelocity ?? 0;
              if (gestureDeltaHorizontal.abs() >= deltaThreshold &&
                  velocity.abs() >= velocityThreshold) {
                if (gestureDeltaHorizontal < 0) {
                  onSwipeRight?.call();
                } else {
                  onSwipeLeft?.call();
                }
              }
            },
      onVerticalDragStart: !listenForVerticalGestures
          ? null
          : (details) {
              gestureStartY = details.globalPosition.dy;
            },
      onVerticalDragUpdate: !listenForVerticalGestures
          ? null
          : (details) {
              gestureEndY = details.globalPosition.dy;
              gestureDeltaVertical = gestureEndY - gestureStartY;
            },
      onVerticalDragEnd: !listenForVerticalGestures
          ? null
          : (details) {
              final velocity = details.primaryVelocity ?? 0;
              if (gestureDeltaVertical.abs() >= deltaThreshold &&
                  velocity.abs() >= velocityThreshold) {
                if (gestureDeltaVertical > 0) {
                  onSwipeTop?.call();
                } else {
                  onSwipeBottom?.call();
                }
              }
            },
      onTap: onTap,
      behavior: behavior,
      child: child,
    );
  }
}
