import 'package:flutter/material.dart';

class HintView extends StatelessWidget {
  final String text;
  final Color? textColor;
  final double textSize;
  final IconData iconData;
  final Color? iconDataColor;
  final double iconDataSize;
  final Widget? textView;
  final Widget? iconView;
  final EdgeInsets padding;
  final double dividerHeight;
  final double? width;
  final double? height;

  const HintView({
    Key? key,
    String? text,
    this.textColor,
    double? textSize,
    IconData? iconData,
    this.iconDataColor,
    double? iconDataSize,
    this.textView,
    this.iconView,
    EdgeInsets? padding,
    double? dividerHeight,
    this.width,
    this.height,
  })  : text = text ?? 'No data to show',
        textSize = textSize ?? 22,
        iconData = iconData ?? Icons.sentiment_dissatisfied,
        iconDataSize = iconDataSize ?? 96,
        padding = padding ?? const EdgeInsets.all(16),
        dividerHeight = dividerHeight ?? 16,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      width: width,
      height: height,
      alignment: Alignment.center,
      padding: padding,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          _getIcon(context),
          SizedBox(
            height: dividerHeight,
          ),
          _getText(context),
        ],
      ),
    );
  }

  Widget _getIcon(BuildContext context) {
    if (iconView != null) return iconView!;
    final iconDataColor =
        this.iconDataColor ?? Theme.of(context).textTheme.headlineMedium?.color;
    return Icon(
      iconData,
      size: iconDataSize,
      color: iconDataColor,
    );
  }

  Widget _getText(BuildContext context) {
    if (textView != null) return textView!;
    final textColor =
        this.textColor ?? Theme.of(context).textTheme.headlineMedium?.color;
    return Center(
        child: Text(
      text,
      style: TextStyle(color: textColor, fontSize: textSize),
      textAlign: TextAlign.center,
    ));
  }
}
