import 'package:flutter/material.dart';
import 'package:adeptviews/src/preference/preference_service.dart';

// ignore_for_file: must_be_immutable
class PreferenceDialog extends StatefulWidget {
  final String? title;
  final List<Widget> preferences;
  final String? submitText;
  final String? cancelText;
  double? textScaleFactor;

  final bool onlySaveOnSubmit;

  PreferenceDialog(this.preferences,
      {this.title,
      this.submitText,
      this.onlySaveOnSubmit = false,
      this.cancelText,
      this.textScaleFactor});

  @override
  PreferenceDialogState createState() => PreferenceDialogState();
}

class PreferenceDialogState extends State<PreferenceDialog> {
  @override
  void initState() {
    super.initState();

    if (widget.onlySaveOnSubmit) {
      PrefService.rebuildCache();
      PrefService.enableCaching();
    }
  }

  @override
  void dispose() {
    PrefService.disableCaching();
    PrefService.rebuildCache();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final dialog = AlertDialog(
      title: widget.title == null ? null : Text(widget.title!),
      content: FutureBuilder(
        future: PrefService.init(),
        builder: (context, snapshot) {
          if (!snapshot.hasData) return const SizedBox();

          return SingleChildScrollView(
            child: Column(
              children: widget.preferences,
            ),
          );
        },
      ),
      actions: <Widget>[
        ...widget.cancelText == null
            ? []
            : [
                TextButton(
                  child: Text(widget.cancelText!),
                  onPressed: () {
                    Navigator.of(context).pop();
                  },
                )
              ],
        ...widget.submitText == null
            ? []
            : [
                TextButton(
                  child: Text(widget.submitText!),
                  onPressed: () {
                    if (widget.onlySaveOnSubmit) {
                      PrefService.applyCache();
                    }
                    Navigator.of(context).pop();
                  },
                )
              ]
      ],
    );
    return widget.textScaleFactor != null
        ? MediaQuery(
            data: MediaQuery.of(context)
                .copyWith(textScaleFactor: widget.textScaleFactor),
            child: dialog,
          )
        : dialog;
  }
}
