import 'package:flutter/material.dart';
import 'package:adeptviews/src/preference/preference_service.dart';

class SwitchPreference extends StatefulWidget {
  final String title;
  final String? summary;
  final String? summaryPrefix;
  final String? summarySuffix;
  final String localKey;
  final bool defaultValue;
  final Widget? leading;
  final bool enabled;
  final bool useValueAsSummary;
  final ValueChanged<bool>? onChange;

  const SwitchPreference(
    this.title,
    this.localKey, {
    Key? key,
    this.summary,
    this.summaryPrefix,
    this.summarySuffix,
    this.defaultValue = false,
    this.leading,
    this.enabled = true,
    this.useValueAsSummary = false,
    this.onChange,
  }) : super(key: key);

  @override
  SwitchPreferenceState createState() => SwitchPreferenceState();
}

class SwitchPreferenceState extends State<SwitchPreference> {
  late String summary;
  late bool value;

  @override
  void initState() {
    super.initState();
    value = PrefService.getBool(widget.localKey) ?? widget.defaultValue;
    summary = '${(widget.summaryPrefix ?? '')}'
        '${(widget.useValueAsSummary ? value : (widget.summary ?? ''))}'
        '${(widget.summarySuffix ?? '')}';
  }

  @override
  Widget build(BuildContext context) {
    return ListTile(
      title: Text(widget.title),
      subtitle: summary.isNotEmpty ? Text(summary) : null,
      leading: widget.leading,
      enabled: widget.enabled,
      onTap: () => onChange(!value),
      trailing: Switch(
        value: value,
        onChanged: (value) => onChange(value),
      ),
    );
  }

  void onChange(value) {
    widget.onChange?.call(value);
    PrefService.setBool(widget.localKey, this.value = value);
    setState(() {});
  }
}
