import 'package:flutter/material.dart';

class Badge extends StatelessWidget {
  final EdgeInsets padding;
  final Radius radius;
  final Color badgeColor;

  final String text;
  final TextStyle textStyle;

  final InlineSpan? textSpan;
  final TextStyle? style;
  final StrutStyle? strutStyle;
  final TextAlign? textAlign;
  final TextDirection? textDirection;
  final Locale? locale;
  final bool? softWrap;
  final TextOverflow? overflow;
  final double? textScaleFactor;
  final int? maxLines;
  final String? semanticsLabel;
  final TextWidthBasis? textWidthBasis;
  final Text? textView;

  Badge({
    Key? key,
    String? text,
    Color? badgeColor,
    Color? textColor,
    double? fontSize,
    this.padding = const EdgeInsets.only(top: 2, right: 4, bottom: 2, left: 4),
    this.radius = const Radius.circular(8),
    TextStyle? textStyle,
    this.textSpan,
    this.style,
    this.strutStyle,
    this.textAlign,
    this.textDirection,
    this.locale,
    this.softWrap,
    this.overflow,
    this.textScaleFactor,
    this.maxLines,
    this.semanticsLabel,
    this.textWidthBasis,
    this.textView,
  })  : text = text ?? '',
        textStyle = textStyle ??
            TextStyle(
                color: textColor ?? Colors.white, fontSize: fontSize ?? 10),
        badgeColor = badgeColor ?? Colors.blue,
        super(key: key);

  @override
  Widget build(BuildContext context) => Container(
        padding: padding,
        decoration: BoxDecoration(
          color: badgeColor,
          borderRadius: BorderRadius.all(radius),
        ),
        child: textView ??
            Text(
              text,
              maxLines: maxLines,
              style: textStyle,
              strutStyle: strutStyle,
              textAlign: textAlign,
              textDirection: textDirection,
              locale: locale,
              softWrap: softWrap,
              overflow: overflow,
              textScaleFactor: textScaleFactor,
              semanticsLabel: semanticsLabel,
              textWidthBasis: textWidthBasis,
            ),
      );
}
