import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:adeptutils/adeptutils.dart';

class NoKeyboardEditableText extends EditableText {
  NoKeyboardEditableText({
    Key? key,
    TextEditingController? controller,
    FocusNode? focusNode,
    TextStyle style = const TextStyle(color: Colors.black),
    Color cursorColor = Colors.black,
    bool autofocus = false,
    Color? selectionColor,
    ValueChanged<String>? onChanged,
    VoidCallback? onEditingComplete,
    ValueChanged<String>? onSubmitted,
    SelectionChangedCallback? onSelectionChanged,
  }) : super(
          key: key,
          controller: controller ?? TextEditingController(),
          focusNode: focusNode ?? NoKeyboardEditableTextFocusNode(),
          backgroundCursorColor: Colors.grey,
          style: style,
          cursorColor: cursorColor,
          autofocus: autofocus,
          selectionColor: selectionColor,
          onChanged: onChanged,
          onEditingComplete: onEditingComplete,
          onSubmitted: onSubmitted,
          onSelectionChanged: onSelectionChanged,
        );

  @override
  EditableTextState createState() {
    return NoKeyboardEditableTextState();
  }
}

class NoKeyboardEditableTextState extends EditableTextState {
  FocusNode get focusNode => widget.focusNode;
  @override
  void requestKeyboard() {
    //super.requestKeyboard(); // don't call the super method
    //just request focus
    focusNode.requestFocus();
  }
}

class NoKeyboardEditableTextFocusNode extends FocusNode {
  @override
  bool consumeKeyboardToken() {
    /// Prevents virtual keyboard from showing on first focus.
    /// Only for Android because on iOS when virtual keyboard is hidden then
    /// the onChange events on the InputField are not fired.
    /// Anyway it doesn't matter the keyboard to be shown because this feature
    /// is ued by the DataWedgeModes which depends on a external device which
    /// works as an external physical keyboard and then virtual keyboard
    /// will be hidden by default.

    if (PlatformUtils.isAndroid) {
      Future.delayed(const Duration(milliseconds: 10),
          () => SystemChannels.textInput.invokeMethod('TextInput.hide'));
    }
    return true;
  }
}
