import 'dart:async';

import 'package:adeptviews/src/text/timer/clip_digit.dart';
import 'package:adeptviews/src/text/timer/slide_direction.dart';
import 'package:flutter/widgets.dart';

typedef DigitFunction = int Function(DateTime? dateTime);

class Digit extends StatefulWidget {
  final Stream<int> itemStream;
  final int initValue;
  final TextStyle? textStyle;
  final BoxDecoration? decoration;
  final SlideDirection slideDirection;
  final EdgeInsets? padding;

  const Digit({
    required this.itemStream,
    this.initValue = 0,
    this.textStyle,
    this.decoration,
    this.slideDirection = SlideDirection.DOWN,
    this.padding,
  });

  @override
  DigitState createState() => DigitState();
}

class DigitState extends State<Digit> with SingleTickerProviderStateMixin {
  late StreamSubscription<int> _streamSubscription;
  int _currentValue = 0;
  int _nextValue = 0;
  late AnimationController _controller;

  bool haveData = false;

  late Animation<Offset> _slideDownAnimation;
  final Animatable<Offset> _slideDownDetails = Tween<Offset>(
    begin: const Offset(0.0, -1.0),
    end: Offset.zero,
  );

  late Animation<Offset> _slideDownAnimation2;
  final Animatable<Offset> _slideDownDetails2 = Tween<Offset>(
    begin: const Offset(0.0, 0.0),
    end: const Offset(0.0, 1.0),
  );

  @override
  void initState() {
    super.initState();
    _controller = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 450));
    _slideDownAnimation = _controller.drive(_slideDownDetails);
    _slideDownAnimation2 = _controller.drive(_slideDownDetails2);

    _controller.addStatusListener((status) {
      if (status == AnimationStatus.completed) {
        _controller.reset();
      }

      if (status == AnimationStatus.dismissed) {
        _currentValue = _nextValue;
      }
    });

    _currentValue = widget.initValue;
    _streamSubscription = widget.itemStream.distinct().listen((value) {
      haveData = true;
      if (value != _currentValue) {
        _nextValue = value;
        _controller.forward();
      }
    });
  }

  @override
  void dispose() {
    _controller.dispose();
    _streamSubscription.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final fakeWidget = Opacity(
      opacity: 0.0,
      child: Text(
        '9',
        style: widget.textStyle,
        textScaleFactor: 1.0,
        textAlign: TextAlign.center,
      ),
    );

    return Container(
      padding: widget.padding,
      alignment: Alignment.center,
      decoration: widget.decoration ?? const BoxDecoration(),
      child: AnimatedBuilder(
        animation: _controller,
        builder: (context, w) {
          return Stack(
            fit: StackFit.passthrough,
            clipBehavior: Clip.hardEdge,
            // overflow: Overflow.clip,
            children: <Widget>[
              haveData
                  ? FractionalTranslation(
                      translation:
                          (widget.slideDirection == SlideDirection.DOWN)
                              ? _slideDownAnimation.value
                              : -_slideDownAnimation.value,
                      child: ClipRect(
                        clipper: ClipHalfRect(
                          percentage: _slideDownAnimation.value.dy,
                          isUp: true,
                          slideDirection: widget.slideDirection,
                        ),
                        child: Text(
                          '$_nextValue',
                          textAlign: TextAlign.center,
                          textScaleFactor: 1.0,
                          style: widget.textStyle,
                        ),
                      ),
                    )
                  : const SizedBox(),
              FractionalTranslation(
                translation: (widget.slideDirection == SlideDirection.DOWN)
                    ? _slideDownAnimation2.value
                    : -_slideDownAnimation2.value,
                child: ClipRect(
                  clipper: ClipHalfRect(
                    percentage: _slideDownAnimation2.value.dy,
                    isUp: false,
                    slideDirection: widget.slideDirection,
                  ),
                  child: Text(
                    '$_currentValue',
                    textAlign: TextAlign.center,
                    textScaleFactor: 1.0,
                    style: widget.textStyle,
                  ),
                ),
              ),
              fakeWidget,
            ],
          );
        },
      ),
    );
  }
}
